@@ -9,9 +9,9 @@ import sys
 import types  # The types line can eventually be removed
 import warnings
 
-from bn import AttributeDict, uniform_path
+from bn import AttributeDict, uniform_path, OrderedDict
 from commandtool import handle_command, Cmd, LoggingCmd
-from configconvert import parse_and_split_options_change_dir
+from configconvert import parse_and_split_options_change_dir, parse_config_string, split_options
 from configconvert.internal import eval_import
 from nestedrecord import encode
 from pipestack import __version__
@@ -77,9 +77,10 @@ class AppCmd(Cmd):
         ),
     })
 
-    def on_run(self, app, args, opts):
-        app.parse_config(args, opts)
-        app.parse_logging(args, opts)
+    def run(self, cmd):
+        return 0
+        self.bag.app.parse_config(cmd.args, cmd.opts)
+        self.bag.app.parse_logging(cmd.args, cmd.opts)
 
 #
 # Core classes
@@ -96,11 +97,15 @@ class App(object):
 
     def __init__(self, option=None):
         self._pipes_by_name = {}
-        self._commands_structure = {None: AppCmd()}
+        self._plugins = {}
+        self._commands_structure = OrderedDict()
         self._instantiated_pipes = {}
         self.config = AttributeDict()
         self.config_file = None
-        self.option = option
+        if isinstance(option, (str, unicode)):
+            self.option = split_options(parse_config_string(option))
+        else:
+            self.option = option
         self.plugins = {}
 
         pipes = self.pipes[:]
@@ -139,13 +144,10 @@ class App(object):
                 "You cannot have two commands named %r"%(command.name,)
             )
         if isinstance(command.class_, (str, unicode)):
-            class_ = eval_import(command.class_)
-        else:
-            class_ = command.class_
-        if isinstance(class_, dict):
-            self._commands_structure[command.name] = class_
-        else:
-            self._commands_structure[command.name] = class_(command.name, command.aliases, **command.extras)
+            command['class_'] = eval_import(command.class_)
+        elif isinstance(command.class_, (tuple, list)):
+            command['class_'] = OrderedDict(command.class_)
+        self._commands_structure[command.name] = command
  
     def _get_default_pipeline(self):
         exclude = getattr(self, 'exclude', ())
@@ -159,16 +161,42 @@ class App(object):
         return pipeline
     default_pipeline = property(_get_default_pipeline)
 
-    def handle_command_line(self, cmd_line_parts=None, exit=sys.exit):
+    def handle_command_line(self, cmd_line_parts=None, program=None, exit=sys.exit, service=None):
         if not self._commands_structure:
             raise Exception('No commands specified in the app')
         else:
-            result = handle_command(
-                self._commands_structure, 
-                service=self, 
-                cmd_line_parts=cmd_line_parts,
-            )
-            exit(result)
+            result = []
+            def run(bag):
+                def initialise_branch(bag, command_tree, command_structure):
+                    # Iterate through the entire structure to initialise each one
+                    for name, command in command_structure.items():
+                        if hasattr(command, 'class_'):
+                            if isinstance(command.class_, dict):
+                                command_tree[name] = OrderedDict()
+                                initialise_branch(bag, command_tree[name], command.class_)
+                            else:
+                                command_tree[name] = command.class_(bag=bag, name=name, aliases=command.aliases)
+                        else:
+                            if hasattr(command, 'class_'):
+                                command_tree[name] = OrderedDict()
+                                initialise_branch(bag, command_tree[name], command)
+                            elif isinstance(command, Cmd):
+                                # Already initialised
+                                command_tree[name] = command
+                            else:# isinstance(command, Cmd):
+                                command_tree[name] = command(bag=bag, name=name)
+                            #else:
+                            #    raise Exception('Unknown command type %r'%command)
+                command_tree = OrderedDict([(None, AppCmd(bag))])
+                initialise_branch(bag, command_tree, self._commands_structure)
+                result.append(handle_command(
+                    command_tree,
+                    program=program,
+                    service=service,
+                    cmd_line_parts=cmd_line_parts,
+                ))
+            self.start_flow(bag=None, ensure=[], run=run)
+            exit(result[0])
 
     def start_flow(self, bag=None, ensure=[], run=None):
         for name in ensure:
