@@ -5,55 +5,10 @@ Base classes for the different types of Pipe
 import getopt
 import logging
 import warnings
-from bn import AttributeDict
+from bn import AttributeDict, MarbleLike
 
 log = logging.getLogger(__name__)
 
-#class Pipe(object):
-#    """\
-#    Pipes can be implemented using classes as well as nested functions. 
-#
-#    This ``Pipe`` class is designed to be used as a base class for other 
-#    classes. The derived classes should implement the ``enter()`` and 
-#    ``leave()`` methods.
-#
-#    Pipes are used in a ``Site`` object combined with the ``pipe()``  
-#    function. They each have a name which is passed to ``pipe()`` and 
-#    can specify some aliases for other pipes they use. The aliases
-#    spcified in the derived pipe's ``default_aliases`` member are used
-#    unless an ``aliases`` argument is given to ``pipe()``.
-#
-#    Any extra arguments to ``pipe()`` should be keyword arguments and
-#    can be handled by overriding the ``on_init_extras()`` method in the
-#    derived pipe. By default they are ignored.
-#    """
-#    default_aliases = {}
-#    version = (0,1,0)
-#
-#    # The app would need to keep an install log.
-#    # Suggest app.install_log = /path/to/install/log
-#    # Then default_pipeline would be changed to flow_pipeline
-#
-#    def __init__(self, bag, name, aliases=None, **pextras):
-#        # Don't keep a reference to the bag, just the name this pipe from the ``.name`` attribute given
-#        self.name = name
-#        self.aliases = AttributeDict(self.default_aliases)
-#        if aliases:
-#            self.aliases.update(aliases)
-#        self.persistent_state = None
-#        self.parse_options(bag)
-#
-#    # Should not add other marbles to the bag
-#    def parse_options(self, bag):
-#        pass
-#
-#
-#    def enter(self, bag):
-#        log.debug('The enter() method hasn\'t been overidden in the %r derived class', self.__class__.__name__)
-#
-#    def leave(self, bag, error=False):
-#        log.debug('The leave() method hasn\'t been overidden in the %r derived class', self.__class__.__name__)
-
 class Pipe(object):
     """\
     Like a config pipe but with support for basic configuration options in the form of Fields.
@@ -285,7 +240,7 @@ class Pipe(object):
     def uninstall(self, bag):
         pass
 
-class Marble(object):
+class Marble(MarbleLike):
     def __init__(
         marble, 
         bag, 
@@ -296,9 +251,7 @@ class Marble(object):
         persistent_state=None, 
         flow_state=None
     ):
-        marble.__dict__['bag'] = bag
-        marble.__dict__['name'] = name
-        marble.__dict__['aliases'] = aliases
+        MarbleLike.__init__(marble, bag, name, aliases)
         marble.__dict__['default_aliases'] = default_aliases
         marble.__dict__['config'] = config
         marble.__dict__['persistent_state'] = persistent_state
@@ -321,47 +274,7 @@ class Marble(object):
             warnings.warn('The use of on_set_flow_state() is deprecated, please just overide __init__() in %s'%marble.__class__.__name__)
             marble.on_set_flow_state(flow_state)
 
-    def __getattr__(marble, name):
-        if name in marble.__dict__:
-            return marble.__dict__[name]
-        if not marble.__dict__.has_key('aliases'):
-            raise AttributeError('Attempted to access the %r attribute, but the marble is not initialised yet so no aliases are present'%name)
-        if name in marble.aliases:
-            if not name in marble.__dict__:
-                alias = marble.aliases[name]
-                if not marble.bag.has_key(alias):
-                    marble.bag.enter(alias)
-                return marble.bag[alias]
-        log.debug(
-            'Couldn\'t find the pipe %r in marble %r with aliases %r', 
-            name, 
-            marble.__class__.__name__,
-            marble.aliases,
-        )
-        raise AttributeError('No such attribute %r'%name)
-
 class DispatchMarble(Marble):
-#    def __init__(
-#        marble, 
-#        bag, 
-#        name, 
-#        config, 
-#        aliases=None, 
-#        default_aliases=None, 
-#        persistent_state=None, 
-#        flow_state=None
-#    ):
-#        Marble.__init__(
-#            marble, 
-#            bag,
-#            name,
-#            config,
-#            aliases,
-#            default_aliases,
-#            persistent_state,
-#            flow_state,
-#        )
-#        marble.dispatch()
           
     def dispatch(marble):
         pass
